"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const js_sdk_common_1 = require("@launchdarkly/js-sdk-common");
const serialization_1 = require("../store/serialization");
/**
 * @internal
 */
class PollingProcessorFDv2 {
    constructor(_requestor, _pollInterval = 30, _logger) {
        this._requestor = _requestor;
        this._pollInterval = _pollInterval;
        this._logger = _logger;
        this._stopped = false;
    }
    _poll(dataCallback, statusCallback) {
        var _a;
        if (this._stopped) {
            return;
        }
        const startTime = Date.now();
        (_a = this._logger) === null || _a === void 0 ? void 0 : _a.debug('Polling LaunchDarkly for feature flag updates');
        this._requestor.requestAllData((err, body) => {
            var _a, _b, _c, _d, _e;
            if (this._stopped) {
                return;
            }
            const elapsed = Date.now() - startTime;
            const sleepFor = Math.max(this._pollInterval * 1000 - elapsed, 0);
            (_a = this._logger) === null || _a === void 0 ? void 0 : _a.debug('Elapsed: %d ms, sleeping for %d ms', elapsed, sleepFor);
            if (err) {
                const { status } = err;
                if (status && !(0, js_sdk_common_1.isHttpRecoverable)(status)) {
                    const message = (0, js_sdk_common_1.httpErrorMessage)(err, 'polling request');
                    (_b = this._logger) === null || _b === void 0 ? void 0 : _b.error(message);
                    statusCallback(js_sdk_common_1.subsystem.DataSourceState.Closed, new js_sdk_common_1.LDPollingError(js_sdk_common_1.DataSourceErrorKind.ErrorResponse, message, status, false));
                    // It is not recoverable, return and do not trigger another poll.
                    return;
                }
                const message = (0, js_sdk_common_1.httpErrorMessage)(err, 'polling request', 'will retry');
                statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted, new js_sdk_common_1.LDPollingError(js_sdk_common_1.DataSourceErrorKind.ErrorResponse, message, status));
                (_c = this._logger) === null || _c === void 0 ? void 0 : _c.warn(message);
                // schedule poll
                this._timeoutHandle = setTimeout(() => {
                    this._poll(dataCallback, statusCallback);
                }, sleepFor);
                return;
            }
            if (!body) {
                statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted, new js_sdk_common_1.LDPollingError(js_sdk_common_1.DataSourceErrorKind.ErrorResponse, 'Response missing body, will retry.'));
                // schedule poll
                this._timeoutHandle = setTimeout(() => {
                    this._poll(dataCallback, statusCallback);
                }, sleepFor);
                return;
            }
            try {
                const parsed = JSON.parse(body);
                const payloadProcessor = new js_sdk_common_1.internal.PayloadProcessor({
                    flag: (flag) => {
                        (0, serialization_1.processFlag)(flag);
                        return flag;
                    },
                    segment: (segment) => {
                        (0, serialization_1.processSegment)(segment);
                        return segment;
                    },
                }, (errorKind, message) => {
                    statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted, new js_sdk_common_1.LDPollingError(errorKind, message));
                }, this._logger);
                payloadProcessor.addPayloadListener((payload) => {
                    dataCallback(payload.basis, payload);
                });
                payloadProcessor.processEvents(parsed.events);
                // TODO: SDK-855 implement blocking duplicate data source state events in DataAvailability API
                statusCallback(js_sdk_common_1.subsystem.DataSourceState.Valid);
            }
            catch (_f) {
                // We could not parse this JSON. Report the problem and fallthrough to
                // start another poll.
                (_d = this._logger) === null || _d === void 0 ? void 0 : _d.error('Response contained invalid data');
                (_e = this._logger) === null || _e === void 0 ? void 0 : _e.debug(`${err} - Body follows: ${body}`);
                statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted, new js_sdk_common_1.LDPollingError(js_sdk_common_1.DataSourceErrorKind.InvalidData, 'Malformed data in polling response'));
            }
            // schedule poll
            this._timeoutHandle = setTimeout(() => {
                this._poll(dataCallback, statusCallback);
            }, sleepFor);
        });
    }
    start(dataCallback, statusCallback) {
        this._statusCallback = statusCallback; // hold reference for usage in stop()
        statusCallback(js_sdk_common_1.subsystem.DataSourceState.Initializing);
        this._poll(dataCallback, statusCallback);
    }
    stop() {
        var _a;
        if (this._timeoutHandle) {
            clearTimeout(this._timeoutHandle);
            this._timeoutHandle = undefined;
        }
        (_a = this._statusCallback) === null || _a === void 0 ? void 0 : _a.call(this, js_sdk_common_1.subsystem.DataSourceState.Closed);
        this._stopped = true;
        this._statusCallback = undefined;
    }
}
exports.default = PollingProcessorFDv2;
//# sourceMappingURL=PollingProcessorFDv2.js.map