"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const js_sdk_common_1 = require("@launchdarkly/js-sdk-common");
const serialization_1 = require("../store/serialization");
class StreamingProcessorFDv2 {
    constructor(clientContext, streamUriPath, parameters, baseHeaders, _diagnosticsManager, _streamInitialReconnectDelay = 1) {
        this._diagnosticsManager = _diagnosticsManager;
        this._streamInitialReconnectDelay = _streamInitialReconnectDelay;
        const { basicConfiguration, platform } = clientContext;
        const { logger } = basicConfiguration;
        const { requests } = platform;
        this._headers = Object.assign({}, baseHeaders);
        this._logger = logger;
        this._requests = requests;
        this._streamUri = (0, js_sdk_common_1.getStreamingUri)(basicConfiguration.serviceEndpoints, streamUriPath, parameters);
    }
    _logConnectionAttempt() {
        this._connectionAttemptStartTime = Date.now();
    }
    _logConnectionResult(success) {
        if (this._connectionAttemptStartTime && this._diagnosticsManager) {
            this._diagnosticsManager.recordStreamInit(this._connectionAttemptStartTime, !success, Date.now() - this._connectionAttemptStartTime);
        }
        this._connectionAttemptStartTime = undefined;
    }
    /**
     * This is a wrapper around the passed errorHandler which adds additional
     * diagnostics and logging logic.
     *
     * @param err The error to be logged and handled.
     * @return boolean whether to retry the connection.
     *
     * @private
     */
    _retryAndHandleError(err, statusCallback) {
        var _a, _b;
        if (!(0, js_sdk_common_1.shouldRetry)(err)) {
            (_a = this._logger) === null || _a === void 0 ? void 0 : _a.error((0, js_sdk_common_1.httpErrorMessage)(err, 'streaming request'));
            this._logConnectionResult(false);
            statusCallback(js_sdk_common_1.subsystem.DataSourceState.Closed, new js_sdk_common_1.LDStreamingError(js_sdk_common_1.DataSourceErrorKind.ErrorResponse, err.message, err.status, false));
            return false;
        }
        (_b = this._logger) === null || _b === void 0 ? void 0 : _b.warn((0, js_sdk_common_1.httpErrorMessage)(err, 'streaming request', 'will retry'));
        this._logConnectionResult(false);
        this._logConnectionAttempt();
        statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted);
        return true;
    }
    start(dataCallback, statusCallback) {
        this._logConnectionAttempt();
        statusCallback(js_sdk_common_1.subsystem.DataSourceState.Initializing);
        const eventSource = this._requests.createEventSource(this._streamUri, {
            headers: this._headers,
            errorFilter: (error) => this._retryAndHandleError(error, statusCallback),
            initialRetryDelayMillis: 1000 * this._streamInitialReconnectDelay,
            readTimeoutMillis: 5 * 60 * 1000,
            retryResetIntervalMillis: 60 * 1000,
        });
        this._eventSource = eventSource;
        const payloadReader = new js_sdk_common_1.internal.PayloadStreamReader(eventSource, {
            flag: (flag) => {
                (0, serialization_1.processFlag)(flag);
                return flag;
            },
            segment: (segment) => {
                (0, serialization_1.processSegment)(segment);
                return segment;
            },
        }, (errorKind, message) => {
            statusCallback(js_sdk_common_1.subsystem.DataSourceState.Interrupted, new js_sdk_common_1.LDStreamingError(errorKind, message));
            // parsing error was encountered, defensively close the data source
            this.stop();
        }, this._logger);
        payloadReader.addPayloadListener((payload) => {
            this._logConnectionResult(true);
            dataCallback(payload.basis, payload);
        });
        eventSource.onclose = () => {
            var _a;
            (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info('Closed LaunchDarkly stream connection');
            statusCallback(js_sdk_common_1.subsystem.DataSourceState.Closed);
        };
        eventSource.onerror = () => {
            // The work is done by `errorFilter`.
        };
        eventSource.onopen = () => {
            var _a;
            (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info('Opened LaunchDarkly stream connection');
            statusCallback(js_sdk_common_1.subsystem.DataSourceState.Valid);
        };
        eventSource.onretrying = (e) => {
            var _a;
            (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info(`Will retry stream connection in ${e.delayMillis} milliseconds`);
        };
    }
    stop() {
        var _a;
        (_a = this._eventSource) === null || _a === void 0 ? void 0 : _a.close();
        this._eventSource = undefined;
    }
    close() {
        this.stop();
    }
}
exports.default = StreamingProcessorFDv2;
//# sourceMappingURL=StreamingProcessorFDv2.js.map